/**
 * DBEC-GP-OMP-CUDA-MPI programs are developed by:
 *
 * Vladimir Loncar, Antun Balaz
 * (Scientific Computing Laboratory, Institute of Physics Belgrade, Serbia)
 *
 * Srdjan Skrbic
 * (Department of Mathematics and Informatics, Faculty of Sciences, University of Novi Sad, Serbia)
 *
 * Paulsamy Muruganandam
 * (Bharathidasan University, Tamil Nadu, India)
 *
 * Luis E. Young-S, Sadhan K. Adhikari
 * (UNESP - Sao Paulo State University, Brazil)
 *
 *
 * Public use and modification of these codes are allowed provided that the
 * following papers are cited:
 * [1] V. Loncar et al., Comput. Phys. Commun. 209 (2016) 190.      
 * [2] V. Loncar et al., Comput. Phys. Commun. 200 (2016) 406.      
 * [3] R. Kishor Kumar et al., Comput. Phys. Commun. 195 (2015) 117.
 *
 * The authors would be grateful for all information and/or comments
 * regarding the use of the programs.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fftw3.h>
#include <omp.h>
#include <sys/time.h>

#define MAX(a, b)             (a > b) ? a : b
#define MAX_FILENAME_SIZE     256
#define RMS_ARRAY_SIZE        3

#define BOHR_RADIUS           5.2917720859e-11
#define XI_LIMIT              26.

#define FFT_FLAG FFTW_MEASURE
/* #define FFT_FLAG FFTW_ESTIMATE */


char *output, *rmsout, *initout, *Nstpout, *Npasout, *Nrunout;
long outstpz, outstpx;

int opt;
long Na;
long Nstp, Npas, Nrun;
long Nz, Nx;
long Nz2, Nx2;
double g, g0, gd, gd0, g_2d, gd_2d, d_y;
double aho, as, add;
double dz, dx;
double dz2, dx2;
double dt;
double vgamma, vlambda;
double par;
double pi;

double *z, *x;
double *z2, *x2;
double **psi;
double **psidd2;
double **pot;
double **potdd;

double Az0, Ax0, Az0r, Ax0r, Az, Ax;
double *calphaz, *calphax;
double *cgammaz, *cgammax;

fftw_plan plan_forward, plan_backward;

void readpar(void);
void initpsi(double **);
void initpot(void);
void gencoef(void);
void initpotdd(double *, double *, double *, double *);
void calcnorm(double *, double **, double **, double **);
void calcmuen(double *, double *, double **, double **, double **, double **, double **, double **, double **);
void calcpsidd2(double **, double **);
void calcrms(double *, double **, double **, double **);
void calcnu(double **, double **);
void calcluz(double **, double **);
void calclux(double **, double **);
void outpsi2xz(double **, FILE *);
void outdenz(double **, double *, FILE *);
void outdenx(double **, double *, FILE *);

extern double simpint(double, double *, long);
extern void diff(double, double *, double *, long);

extern int cfg_init(char *);
extern char *cfg_read(char *);

extern double *alloc_double_vector(long);
extern double **alloc_double_matrix(long, long);
extern void free_double_vector(double *);
extern void free_double_matrix(double **);
