/**
 * DBEC-GP-OMP-CUDA-MPI programs are developed by:
 *
 * Vladimir Loncar, Antun Balaz
 * (Scientific Computing Laboratory, Institute of Physics Belgrade, Serbia)
 *
 * Srdjan Skrbic
 * (Department of Mathematics and Informatics, Faculty of Sciences, University of Novi Sad, Serbia)
 *
 * Paulsamy Muruganandam
 * (Bharathidasan University, Tamil Nadu, India)
 *
 * Luis E. Young-S, Sadhan K. Adhikari
 * (UNESP - Sao Paulo State University, Brazil)
 *
 *
 * Public use and modification of these codes are allowed provided that the
 * following papers are cited:
 * [1] V. Loncar et al., Comput. Phys. Commun. 209 (2016) 190.      
 * [2] V. Loncar et al., Comput. Phys. Commun. 200 (2016) 406.      
 * [3] R. Kishor Kumar et al., Comput. Phys. Commun. 195 (2015) 117.
 *
 * The authors would be grateful for all information and/or comments
 * regarding the use of the programs.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fftw3.h>
#include <complex.h>
#include <omp.h>
#include <sys/time.h>

#define MAX(a, b)             (a > b) ? a : b
#define MAX_FILENAME_SIZE     256
#define RMS_ARRAY_SIZE        3

#define BOHR_RADIUS           5.2917720859e-11

#define XI_LIMIT              26.
#define POTDD_ASYMPTOTIC(a)   2. - 3. * (1. - 1. / (2. * a * a) + 3. / (4. * a * a * a * a) - 15. / (8. * a * a * a * a * a * a))
/* #define POTDD_ASYMPTOTIC(a)   - 1. */

#define FFT_FLAG FFTW_MEASURE
//#define FFT_FLAG FFTW_ESTIMATE

char *output, *rmsout, *initout, *Nstpout, *Npasout, *Nrunout, *dynaout;
long outstpz, outstpx, outstpt;

int opt;
long Na;
long Nstp, Npas, Nrun;
long Nz, Nx;
long Nz2, Nx2;
double g, g0, gpar, gd, gd0, gdpar, g_2d, gd_2d, d_y;
double aho, as, add;
double dz, dx;
double dz2, dx2;
double dt;
double vgamma, vlambda;
double par;
double pi;

double *z, *x;
double *z2, *x2;
double complex **psi;
double **psidd2;
double **pot;
double **potdd;

double complex Az0, Ax0, Az0r, Ax0r, Az, Ax;
double complex *calphaz, *calphax;
double complex *cgammaz, *cgammax;

fftw_plan plan_forward, plan_backward;

void readpar(void);
void initpsi(double complex **);
void initpot(void);
void gencoef(void);
void initpotdd(double *, double *, double *, double *);
void calcnorm(double *, double complex **, double **, double **);
void calcmuen(double *, double *, double complex **, double **, double **, double **, double **, double **, double **);
void calcpsidd2(double complex **, double **);
void calcrms(double *, double complex **, double **, double **);
void calcnu(double complex **, double **);
void calcluz(double complex **, double complex **);
void calclux(double complex **, double complex **);
void outpsi2xz(double complex **, FILE *);
void outdenz(double complex **, double *, FILE *);
void outdenx(double complex **, double *, FILE *);

extern double simpint(double, double *, long);
extern void diff(double, double *, double *, long);

extern int cfg_init(char *);
extern char *cfg_read(char *);

extern double *alloc_double_vector(long);
extern double complex *alloc_complex_vector(long);
extern double **alloc_double_matrix(long, long);
extern double complex **alloc_complex_matrix(long, long);
extern void free_double_vector(double *);
extern void free_complex_vector(double complex *);
extern void free_double_matrix(double **);
extern void free_complex_matrix(double complex **);
