/**
 * DBEC-GP-OMP-CUDA-MPI programs are developed by:
 *
 * Vladimir Loncar, Antun Balaz
 * (Scientific Computing Laboratory, Institute of Physics Belgrade, Serbia)
 *
 * Srdjan Skrbic
 * (Department of Mathematics and Informatics, Faculty of Sciences, University of Novi Sad, Serbia)
 *
 * Paulsamy Muruganandam
 * (Bharathidasan University, Tamil Nadu, India)
 *
 * Luis E. Young-S, Sadhan K. Adhikari
 * (UNESP - Sao Paulo State University, Brazil)
 *
 *
 * Public use and modification of these codes are allowed provided that the
 * following papers are cited:
 * [1] V. Loncar et al., Comput. Phys. Commun. 209 (2016) 190.      
 * [2] V. Loncar et al., Comput. Phys. Commun. 200 (2016) 406.      
 * [3] R. Kishor Kumar et al., Comput. Phys. Commun. 195 (2015) 117.
 *
 * The authors would be grateful for all information and/or comments
 * regarding the use of the programs.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fftw3.h>
#include <complex.h>
#include <omp.h>
#include <sys/time.h>

#define MAX(a, b, c)       (a > b) ? ((a > c) ? a : c) : ((b > c) ? b : c)
#define MAX_FILENAME_SIZE  256
#define RMS_ARRAY_SIZE     4

#define BOHR_RADIUS        5.2917720859e-11

#define FFT_FLAG FFTW_MEASURE
/* #define FFT_FLAG FFTW_ESTIMATE */

char *output, *rmsout, *initout, *Nstpout, *Npasout, *Nrunout, *dynaout;
long outstpx, outstpy, outstpz, outstpt;

int opt;
long Na;
long Nstp, Npas, Nrun;
long Nx, Ny, Nz;
long Nx2, Ny2, Nz2;
double g, g0, gpar, gd, gd0, gdpar;
double aho, as, add;
double dx, dy, dz;
double dx2, dy2, dz2;
double dt;
double vnu, vlambda, vgamma;
double par;
double pi, cutoff;

double *x, *y, *z;
double *x2, *y2, *z2;
double complex ***psi;
double ***psidd2;
double ***pot;
double ***potdd;

double complex Ax0, Ay0, Az0, Ax0r, Ay0r, Az0r, Ax, Ay, Az;
double complex *calphax, *calphay, *calphaz;
double complex *cgammax, *cgammay, *cgammaz;

fftw_plan plan_forward, plan_backward;

void readpar(void);
void initpsi(double complex ***);
void initpot(void);
void gencoef(void);
void initpotdd(double *, double *, double *, double *, double *, double *);
void calcnorm(double *, double complex ***, double **, double **, double **);
void calcmuen(double *, double *, double complex ***, double ***, double ***, double **, double **, double **, double **, double **, double **);
void calcpsidd2(double complex ***, double ***);
void calcrms(double *, double complex ***, double **, double **, double **);
void calcnu(double complex ***, double ***);
void calclux(double complex ***, double complex **);
void calcluy(double complex ***, double complex **);
void calcluz(double complex ***, double complex **);
void outpsi2xy(double complex ***, FILE *);
void outpsi2xz(double complex ***, FILE *);
void outdenx(double complex ***, double *, double *, FILE *);
void outdeny(double complex ***, double *, double *, FILE *);
void outdenz(double complex ***, double *, double *, FILE *);
void outdenxy(double complex ***, double *, FILE *);
void outdenxz(double complex ***, double *, FILE *);
void outdenyz(double complex ***, double *, FILE *);
void outdenxyz(double complex ***, FILE *);

extern double simpint(double, double *, long);
extern void diff(double, double *, double *, long);

extern int cfg_init(char *);
extern char *cfg_read(char *);

extern double *alloc_double_vector(long);
extern double complex *alloc_complex_vector(long);
extern double **alloc_double_matrix(long, long);
extern double complex **alloc_complex_matrix(long, long);
extern double ***alloc_double_tensor(long, long, long);
extern double complex ***alloc_complex_tensor(long, long, long);
extern void free_double_vector(double *);
extern void free_complex_vector(double complex *);
extern void free_double_matrix(double **);
extern void free_complex_matrix(double complex **);
extern void free_double_tensor(double ***);
extern void free_complex_tensor(double complex ***);
